#!/bin/bash

HERE=`pwd`
bindir=${HERE}/bin/
BIN=${bindir}/ISAMIAS.out


np=1
nclones=1
ncpu=$(( $np * nclones ))

modeltype=1     #for the moment 1
samplingtype=1  #should always be 1


# command line input
station=`sed -n '1p' < analysis.txt`
iyear=`sed -n '2p' < analysis.txt`
day1=`sed -n '3p' < analysis.txt`
hour1=`sed -n '4p' < analysis.txt`
day2=`sed -n '5p' < analysis.txt`
hour2=`sed -n '6p' < analysis.txt`
ifreq=`sed -n '7p' < analysis.txt`

#station=$1
#iyear=$2
#day1=$3
#hour1=$4
#day2=$5
#hour2=$6
#ifreq=$7   


iday=${day1}

amiasname=${iyear}_${iday}_${ifreq}_${hour1}-${hour2}

fname1=input1_${amiasname}
fname2=input2_${amiasname}

#sampling data
freq=$ifreq
year1=$iyear ; year2=$iyear
minute1=0 ; minute2=0
nparams=7
nblocks=4
nmoves=1000
nskip=10
nstart=1
#echo $year1 $day1 $hour1 $minute1
#echo $year2 $day2 $hour2 $minute2


#station specific information
#station=PQ052


datadir=${HERE}/${station}/
logo=$station
file=${station}_collectedmaxima_${freq}.dat
dirname=${station}_AMIAS
mkdir -p ${dirname}
mkdir -p ${dirname}/technical
cd ${dirname}


#AMIAS specific ### INPUT FOR GETDATA
datadir0="\"${datadir}\""
cat <<EOF > $fname1
$modeltype   :: type of fit model
$datadir0    :: data dir
$file 
$year1 $day1 $hour1 $minute1
$year2 $day2 $hour2 $minute2
EOF

### INPUT FOR MAIN 
cat <<EOF > $fname2  
$nparams 0       :: nparams
list_params_${amiasname}      :: paramsfile
${nblocks} ${nmoves} ${nskip} ${nstart}     :: nblocks nmoves nskip statistics_start
1  	         :: samplingtype (1->uniform monte-carlo 2->radial monte-carlo 11->uniform (standard amias))
300              :: chi2max
t 1 1.1          :: ptempering,tempering_rate,tempering_factor
f ${nclones}     :: cloning nclones
f                :: writescatter
EOF


cat <<EOF > list_params_${amiasname}
${amiasname}_A0  80. 500.
EOF
lim1=( 0.5 3.0 6.0 )
lim2=( 3.0 8.0 24.0 )
ip=1
ii=0
while [ $ip -lt $nparams ];
do
j=$(( $ii + 1 ))
cat <<EOF >> list_params_${amiasname}
${amiasname}_A${j}  0. 100.
${amiasname}_T${j}  ${lim1[$ii]} ${lim2[$ii]}
${amiasname}_f${j}  0. 7.
EOF
let ip++
let ip++
let ip++
let ii++
done

### FINAL INPUT FILE
cat $fname1 > input_${amiasname}
cat $fname2 >> input_${amiasname}
rm -f $fname1
rm -f $fname2

cd ${HERE}/${dirname}
#echo "now in " $HERE/${dirname}
#echo "executing" $BIN
#mpirun --oversubscribe -np $ncpu $BIN < input | tee ${amiasname}_amias.out
mpirun --oversubscribe -np $ncpu $BIN < input_${amiasname} > ${amiasname}_amias.out
if [ -f ${amiasname}_T1.bin ];
then
mv reconstructed ${amiasname}_reconstructed
gzip -f ${amiasname}*.bin
rm -f chi2.bin
mv -f *.gz technical/
nn=12
a0=`tail -$nn ${amiasname}_amias.out | grep A0 | cut -c60-70`
t1=`tail -$nn ${amiasname}_amias.out | grep T1 | cut -c59-70`
a1=`tail -$nn ${amiasname}_amias.out | grep A1 | cut -c60-70`
t2=`tail -$nn ${amiasname}_amias.out | grep T2 | cut -c59-70`
a2=`tail -$nn ${amiasname}_amias.out | grep A2 | cut -c60-70`
mv -f *amias.out technical/
mv -f input_* technical/
mv -f list_params* technical/
echo $year1 $day1 $hour1 $t1 $a1 $t2 $a2
xmgrace ${amiasname}_reconstructed -hdevice PNG -hardcopy -printfile ${amiasname}_reconstructed.png
mv ${amiasname}_reconstructed technical/
else
echo $year1 $day1 $hour1 "NO DATA AVAILABLE"
fi





cd ${HERE}


