module getdata
  USE MPI
  USE global
implicit none

integer,private :: ndata
double precision,private, allocatable :: simdata(:,:)
integer,private :: dimt
real,private :: startread,stopread
character*200,private  :: datafiles,datadir
integer, private :: modeltype
integer,private :: year1,day1,hour1,minute1
integer,private :: year2,day2,hour2,minute2
double precision,private :: sigma,pi
real,private :: timestep
integer, private :: Ly
!real, private :: snorm
!logical, private,allocatable :: skip(:,:)

CONTAINS

!############################## LQCD #####################
  subroutine readdata()
    implicit none
    integer i,icount,it,j,ifile,jfile,n,year,day,hour,minute
    integer istart, istop
    double precision temp,x,dx
    integer, allocatable :: list(:)
 
    print*,'enter modeltype'
    read*,modeltype
    print*,'read',modeltype
    print*,'enter directory where data is located'
    read*,datadir
    print*,'read '//trim(datadir)
    print*,'enter filename,year,yearday,startread, stopread'
    read*,datafiles
    read*,year1,day1,hour1,minute1
    read*,year2,day2,hour2,minute2
    print*,'Will look for the range:'
    print*,year1,day1,hour1,minute1
    print*,year2,day2,hour2,minute2
    Ly=365
    if(mod(year1,4).eq.0)Ly=366

    open (1,file=trim(datadir)//trim(datafiles),Status='OLD',Err=900)
    n=0
    do
       read(1,*,end=100)
       n=n+1
    end do
100 continue
    print*,'number of lines:',n
    allocate (list(n))
    list=0
    rewind(1)
    
    !!! determine which lines in the file contribute !!!WARNING only designed for at most two consecutive years
    ndata=0
    istart=0
    istop=0
    do i=1,n 
       read(1,*)year,day,hour,minute
       if(istart==0)then
          if(year==year1 .and. day==day1 .and. hour==hour1 .and. minute==minute1)istart=1
          if(year==year1 .and. day==day1 .and. hour==hour1 .and. minute.gt.minute1)istart=1
          if(year==year1 .and. day==day1 .and. hour.gt.hour1)istart=1
       end if
       if(istart==1)then
          if(year==year2 .and. day==day2 .and. hour==hour2 .and. minute==minute2)istop=1
          if(year==year2 .and. day==day2 .and. hour==hour2 .and. minute.gt.minute2)istop=1
          if(year==year2 .and. day==day2 .and. hour.gt.hour2)istop=1
          if(year==year2 .and. day.gt.day2 )istop=1
          if(year.gt.year2 )istop=1
       end if
       if(istop==1)exit
       if(istart==1)then
          ndata=ndata+1
          list(i)=1
          print*,year,day,hour,minute       
       end if
    enddo
    if (ndata==0)then
       write(6,*)'datafile ',trim(datafiles),' does not have any values in the requested timerange'
       stoprun=.true.
       return
    end if
    rewind(1)
    allocate (simdata(ndata,3))
    
    print*,'memory allocated for simulation data ' !,maxval(stopread)-minval(startread)+1
    print*,'looking in directory '//trim(datadir)
    print*,'opening file '//trim(datadir)//trim(datafiles)
    it=0
    do i=1,n 
       read(1,*,end=210)year,day,hour,minute,temp,x,dx,timestep
       !print*,year,day,hour,minute,temp,x,dx,timestep
       if(list(i)==1)then
          temp=(itime(year,day,hour,minute)-1.)*timestep/60.
          it=it+1
          simdata(it,1)=temp
          simdata(it,2)=x
          simdata(it,3)=dx
       end if
    end do
210 continue
    !print*,'done'
    close(1) 
    write(6,*)'read ',it,' simulation data:'
    do i=1,it
       print*,simdata(i,1),simdata(i,2),simdata(i,3)
    end do
    print*,'exiting readdata'
    if(it.ne.ndata)stop 'error in readdata'
    deallocate(list)
    return
900 print*,'error opening file '//trim(datadir)//trim(datafiles)
    stop
  end subroutine readdata

   integer function itime(jyear,jday,jhour,jmin)
    implicit none
    integer, intent(in) :: jyear,jday,jhour,jmin
    real x,y,m,h,d
    !integer Ly    
    itime=1
    m=minute1
    h=hour1
    d=day1
    y=year1
    !print*,y,d,h,m
    !print*,jyear,jday,jhour,jmin
    if(jyear==int(y) .and. jday==int(d) .and. jhour==int(h) .and. jmin==int(m))return
    do
       m=m+timestep
       if(m.gt.60.-timestep)then
          m=0.
          h=h+1.
       end if
       if(h.gt.23)then
          h=0.
          d=d+1
       end if
       if(d.gt.Ly)then
          d=1
          y=y+1
          Ly=365
          if(mod(int(y),4).eq.0)Ly=366
       end if
       itime=itime+1
!!$       if(itime.gt.nt.and..not.first)then
!!$          print*,'logical error in itime',y,d,h,m,itime
!!$          stop
!!$       end if
       !print*,y,d,h,m,itime,'-----------',jyear,jday,jhour,jmin
       !print*,m,h,d,y
       if(jyear==int(y) .and. jday==int(d) .and. jhour==int(h) .and. jmin==int(m))exit 
    end do
    return
  end function itime

  real function time(jyear,jday,jhour,jmin)
    implicit none
    integer, intent(in) :: jyear,jday,jhour,jmin
    real x,y,m,h,d
    !integer Ly    
    m=minute1
    h=hour1
    d=day1
    y=year1
    if(jyear==year1)then
       if(jday==day1.and.jhour==hour1)time=(jmin-minute1)/60.
       if(jday==day1.and.jhour.gt.hour1)time=(60.-minute1)/60. + jmin/60. + (jhour-hour1-1.)
       if(jday.gt.day1)time=(60.-minute1)/60.+jmin/60.+(24-hour1-1.)+jhour+(jday-day1-1)*24.
    else !assume that previous day was the last of the previous year
       time=(60.-minute1)/60.+jmin/60.+(24-hour1-1.)+jhour
    end if
    return
  end function time

  
  double precision function chi2() 
    implicit None
    integer i,j,k!,imodel
    real(kind=8) diff,trial,err,av
!!$    logical :: first=.true.
!!$    if(first)then
!!$       sigma=0.
!!$       av=0.
!!$       do i=1,ndata
!!$          av=av+simdata(i,2)
!!$          sigma=sigma+simdata(i,2)**2
!!$       end do
!!$       sigma=sigma/ndata - (av/ndata)**2
    pi=acos(-1.d0)
!!$       first=.false.
!!$    end if
    
    chi2 = 0.0
    !imodel=2
    select case(modeltype)
    case(1) !A0 + \Sum_i A_i sin(2*pi*t/T +phi)
       do  i=1,ndata 
          trial=fitparams(1)%val
          do j=2,nparams,3
!             trial=trial+fitparams(j)%val*sin(2.*pi*simdata(i,1)/fitparams(j+1)%val+fitparams(j+2)%val) 
             trial=trial+fitparams(j)%val*sin(2.*pi*simdata(i,1)/fitparams(j+1)%val+fitparams(j+2)%val) 
          end do
          diff=simdata(i,2)-trial
!!$          if(abs(simdata(i,3)).gt.1.E-7)then
!!$             Chi2 = Chi2 + (diff**2)/simdata(i,3)**2
!!$          else
          Chi2 = Chi2 + (diff**2)/1.**2
!!$          end if
       end do
    case(2)
       do  i=1,ndata
          k=i/(ndata/(nparams-3.))
          k=3+k+1
          if(k.gt.nparams)k=nparams
          trial=fitparams(k)%val
          trial=trial+fitparams(1)%val*sin(2.*pi*simdata(i,1)/fitparams(2)%val+fitparams(3)%val) 
          diff=simdata(i,2)-trial
          if(abs(simdata(i,3)).gt.1.E-7)then
             Chi2 = Chi2 + (diff**2)/simdata(i,3)**2
          else
             Chi2 = Chi2 + (diff**2)/1.**2
          end if
       end do
    case(3)
       do  i=1,ndata
          k=i/(ndata/(nparams-3.))
          k=3+k+1
          if(k.gt.nparams)k=nparams
          trial=fitparams(1)%val
          trial=trial+fitparams(k)%val*sin(2.*pi*simdata(i,1)/fitparams(2)%val+fitparams(3)%val) 
          diff=simdata(i,2)-trial
          if(abs(simdata(i,3)).gt.1.E-7)then
             Chi2 = Chi2 + (diff**2)/simdata(i,3)**2
          else
             Chi2 = Chi2 + (diff**2)/1.**2
          end if
       end do
    case(4)
       do  i=1,ndata 
          trial=fitparams(1)%val
          do j=2,nparams,5
             trial=trial+(fitparams(j)%val*sin(2.*pi*simdata(i,1)/fitparams(j+3)%val+fitparams(j+4)%val)) &
                  *sin(2.*pi*simdata(i,1)/fitparams(j+1)%val+fitparams(j+2)%val) 
          end do
          diff=simdata(i,2)-trial
          if(abs(simdata(i,3)).gt.1.E-7)then
             Chi2 = Chi2 + (diff**2)/simdata(i,3)**2
          else
             Chi2 = Chi2 + (diff**2)/1.**2
          end if
       end do
    end select
    return
  end function chi2

  subroutine special_binning()
    return
  end subroutine special_binning

  
  subroutine special_plotbins()
    implicit none
    double precision :: temp1,temp2,par(nparams),trial
    integer :: i,j,k,itemp
    !return
    !! plot the reconstructed data
    do i=1,nparams
       call MPI_ALLREDUCE( fitparams(i)%av, temp1, 1, MPI_DOUBLE_PRECISION, MPI_SUM,MPI_COMM_WORLD,mpierror)
    !call MPI_ALLREDUCE( fitparams(i)%sigma, temp2, 1, MPI_DOUBLE_PRECISION, MPI_SUM,MPI_COMM_WORLD,mpierror)
       call MPI_ALLREDUCE( fitparams(i)%neval, itemp, 1, MPI_INTEGER, MPI_SUM,MPI_COMM_WORLD,mpierror)
       par(i)=temp1/itemp
    end do
    if(mpiid==0)then
       open(11,file='reconstructed')
       select case(modeltype)
       case(1)
          do  i=1,ndata 
             trial=par(1)
             do j=2,nparams,3
                trial=trial+par(j)*sin(2.*pi*simdata(i,1)/par(j+1)+par(j+2)) 
             end do
             write(11,*)simdata(i,1),trial
          end do
       case(2)
          do  i=1,ndata 
             k=i/(ndata/(nparams-3.))
             k=3+k+1
             if(k.gt.nparams)k=nparams
             trial=par(k)
             trial=trial+par(1)*sin(2.*pi*simdata(i,1)/par(2)+par(3)) 
             write(11,*)simdata(i,1),trial
          end do
       case(3)
          do  i=1,ndata 
             k=i/(ndata/(nparams-3.))
             k=3+k+1
             if(k.gt.nparams)k=nparams
             trial=par(1)
             trial=trial+par(k)*sin(2.*pi*simdata(i,1)/par(2)+par(3)) 
             write(11,*)simdata(i,1),trial
          end do
       case(4)
          do  i=1,ndata 
             trial=par(1)
             do j=2,nparams,5
                trial=trial+(par(j)*sin(2.*pi*simdata(i,1)/par(j+3)+par(j+4))) &
                     *sin(2.*pi*simdata(i,1)/par(j+1)+par(j+2)) 
             end do
             write(11,*)simdata(i,1),trial
          end do
       end select
       write(11,*)
       write(11,*)
       do  i=1,ndata 
          write(11,*)simdata(i,1),simdata(i,2)
       end do
       close(11)
    end if
    Call MPI_BARRIER(MPI_COMM_WORLD, mpierror )
    return
  end subroutine special_plotbins

subroutine bcastdata()
implicit none
integer :: n,i,j,k
if(mpiid==0)print*,'data will be broadcasted to',mpisize
if(mpisize==1)return
CALL MPI_BCAST(modeltype, 1, MPI_INTEGER, 0, MPI_COMM_WORLD, mpierror)
!CALL MPI_BCAST(special, 1, MPI_LOGICAL, 0, MPI_COMM_WORLD, mpierror)
CALL MPI_BCAST(ndata, 1, MPI_INTEGER, 0, MPI_COMM_WORLD, mpierror)
Call MPI_BARRIER(MPI_COMM_WORLD, mpierror )
if(mpiid.ne.0)then
   allocate (simdata(ndata,3)) !,ndata(n,n),skip(n,n))
end if
Call MPI_BARRIER(MPI_COMM_WORLD, mpierror )
CALL MPI_BCAST(simdata(:,1),ndata, MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, mpierror)
CALL MPI_BCAST(simdata(:,2),ndata, MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, mpierror)
CALL MPI_BCAST(simdata(:,3),ndata, MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, mpierror)

if(mpiid==0)print*,'data was broatcasted'

Call MPI_BARRIER(MPI_COMM_WORLD, mpierror )

end subroutine bcastdata


subroutine cleardata()
  implicit none
  integer :: i
  real*8 :: trial
!!$  open(111,file='temp.dat')
!!$  i=1
!!$  write(111,*)i,simdata(1)
!!$  trial=simdata(1)
!!$  do i=2,ndata
!!$     trial=fitparams(2)%val-trial*fitparams(1)%val
!!$     write(111,*)i,trial
!!$  end do
!!$  close(111)
  deallocate(simdata)
end subroutine cleardata


REAL FUNCTION gennor(av,sd)
      implicit real(a-h,o-z)
!!$C**********************************************************************
!!$C
!!$C     REAL FUNCTION GENNOR( AV, SD )
!!$C
!!$C         GENerate random deviate from a NORmal distribution
!!$C
!!$C
!!$C                              Function
!!$C
!!$C
!!$C     Generates a single random deviate from a normal distribution
!!$C     with mean, AV, and standard deviation, SD.
!!$C
!!$C
!!$C                              Arguments
!!$C
!!$C
!!$C     AV --> Mean of the normal distribution.
!!$C                              REAL AV
!!$C
!!$C     SD --> Standard deviation of the normal distribution.
!!$C                              REAL SD
!!$C
!!$C     GENNOR <-- Generated normal deviate.
!!$C                              REAL GENNOR
!!$C
!!$C
!!$C                              Method
!!$C
!!$C
!!$C     Renames SNORM from TOMS as slightly modified by BWB to use RANF
!!$C     instead of SUNIF.
!!$C
!!$C     For details see:
!!$C               Ahrens, J.H. and Dieter, U.
!!$C               Extensions of Forsythe's Method for Random
!!$C               Sampling from the Normal Distribution.
!!$C               Math. Comput., 27,124 (Oct. 1973), 927 - 937.
!!$C
!!$C
!!$C**********************************************************************
!!$C     .. Scalar Arguments ..
      REAL av,sd
!!$C     ..
!!$C     .. External Functions ..
!      REAL snorm
!      EXTERNAL snorm
!!$C     ..
!!$C     .. Executable Statements ..
      gennor = sd*snorm() + av
      RETURN

    END FUNCTION gennor

    REAL FUNCTION snorm()
      implicit real(a-h,o-z)
!!$C**********************************************************************C
!!$C                                                                      C
!!$C                                                                      C
!!$C     (STANDARD-)  N O R M A L  DISTRIBUTION                           C
!!$C                                                                      C
!!$C                                                                      C
!!$C**********************************************************************C
!!$C**********************************************************************C
!!$C                                                                      C
!!$C     FOR DETAILS SEE:                                                 C
!!$C                                                                      C
!!$C               AHRENS, J.H. AND DIETER, U.                            C
!!$C               EXTENSIONS OF FORSYTHE'S METHOD FOR RANDOM             C
!!$C               SAMPLING FROM THE NORMAL DISTRIBUTION.                 C
!!$C               MATH. COMPUT., 27,124 (OCT. 1973), 927 - 937.          C
!!$C                                                                      C
!!$C     ALL STATEMENT NUMBERS CORRESPOND TO THE STEPS OF ALGORITHM 'FL'  C
!!$C     (M=5) IN THE ABOVE PAPER     (SLIGHTLY MODIFIED IMPLEMENTATION)  C
!!$C                                                                      C
!!$C     Modified by Barry W. Brown, Feb 3, 1988 to use RANF instead of   C
!!$C     SUNIF.  The argument IR thus goes away.                          C
!!$C                                                                      C
!!$C**********************************************************************C
!!$C
      DIMENSION a(32),d(31),t(31),h(31)
      integer i
!!$C
!!$C     THE DEFINITIONS OF THE CONSTANTS A(K), D(K), T(K) AND
!!$C     H(K) ARE ACCORDING TO THE ABOVEMENTIONED ARTICLE
!!$C
      DATA a/0.0,.3917609E-1,.7841241E-1,.1177699,.1573107,.1970991,&
          .2372021,.2776904,.3186394,.3601299,.4022501,.4450965,&
          .4887764,.5334097,.5791322,.6260990,.6744898,.7245144,&
          .7764218,.8305109,.8871466,.9467818,1.009990,1.077516,&
          1.150349,1.229859,1.318011,1.417797,1.534121,1.675940,&
          1.862732,2.153875/
      DATA d/5*0.0,.2636843,.2425085,.2255674,.2116342,.1999243, &
          .1899108,.1812252,.1736014,.1668419,.1607967,.1553497,&
          .1504094,.1459026,.1417700,.1379632,.1344418,.1311722,&
          .1281260,.1252791,.1226109,.1201036,.1177417,.1155119,&
          .1134023,.1114027,.1095039/
      DATA t/.7673828E-3,.2306870E-2,.3860618E-2,.5438454E-2, &
          .7050699E-2,.8708396E-2,.1042357E-1,.1220953E-1,.1408125E-1,&
          .1605579E-1,.1815290E-1,.2039573E-1,.2281177E-1,.2543407E-1,&
          .2830296E-1,.3146822E-1,.3499233E-1,.3895483E-1,.4345878E-1,&
          .4864035E-1,.5468334E-1,.6184222E-1,.7047983E-1,.8113195E-1,&
          .9462444E-1,.1123001,.1364980,.1716886,.2276241,.3304980,&
          .5847031/
      DATA h/.3920617E-1,.3932705E-1,.3950999E-1,.3975703E-1,&
          .4007093E-1,.4045533E-1,.4091481E-1,.4145507E-1,.4208311E-1,&
          .4280748E-1,.4363863E-1,.4458932E-1,.4567523E-1,.4691571E-1,&
          .4833487E-1,.4996298E-1,.5183859E-1,.5401138E-1,.5654656E-1,&
          .5953130E-1,.6308489E-1,.6737503E-1,.7264544E-1,.7926471E-1,&
          .8781922E-1,.9930398E-1,.1155599,.1404344,.1836142,.2790016,&
          .7010474/
!!$C
   10 u = random()
      s = 0.0
      IF (u.GT.0.5) s = 1.0
      u = u + u - s
   20 u = 32.0*u
      i = int(u)
      IF (i.EQ.32) i = 31
      IF (i.EQ.0) GO TO 100
!!$C
!!$C                                START CENTER
!!$C
   30 ustar = u - float(i)
      aa = a(i)
   40 IF (ustar.LE.t(i)) GO TO 60
      w = (ustar-t(i))*h(i)
!!$C
!!$C                                EXIT   (BOTH CASES)
!!$C
   50 y = aa + w
      snorm = y
      IF (s.EQ.1.0) snorm = -y
      RETURN
!!$C
!!$C                                CENTER CONTINUED
!!$C
   60 u = random()
      w = u* (a(i+1)-aa)
      tt = (0.5*w+aa)*w
      GO TO 80

   70 tt = u
      ustar = random()
   80 IF (ustar.GT.tt) GO TO 50
   90 u = random()
      IF (ustar.GE.u) GO TO 70
      ustar = random()
      GO TO 40
!!$C
!!$C                                START TAIL
!!$C
  100 i = 6
      aa = a(32)
      GO TO 120

  110 aa = aa + d(i)
      i = i + 1
  120 u = u + u
      IF (u.LT.1.0) GO TO 110
  130 u = u - 1.0
  140 w = u*d(i)
      tt = (0.5*w+aa)*w
      GO TO 160

  150 tt = u
  160 ustar = random()
      IF (ustar.GT.tt) GO TO 50
  170 u = random()
      IF (ustar.GE.u) GO TO 150
      u = random()
      GO TO 140

    END FUNCTION snorm

    
end module getdata
