!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!       
!  
! AMIAS main file version 1.0
!
! 19.8.2017
!
! parallel tempering and/or farming
!
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
USE MPI
USE global
USE getdata
implicit none

!!! general purpose
integer :: i,ii,j,k
logical :: writescatter
character*3 :: ch
double precision :: x

!!! monte carlo
integer :: imove,iblock,iskip,icount,itemp,statistics_start=1
integer :: samplingtype
double precision :: temp1,temp2,chi20,chi2max
logical :: pass

!!! declarations for parallel tempering and cloning
double precision :: tempering_factor
logical :: ptempering,cloning
integer, allocatable :: temperingorder(:)
integer :: mpiid_clone,mpiid_temp
integer :: i1,i2,nmoves_ex,exchange_succ,pass2,imoves_tempering,tempering_rate,ntemperatures,nclones
integer, allocatable :: status1(:,:), requests1(:)
integer, allocatable :: status2(:,:), requests2(:)
integer :: comm_clone

!!! timing
real :: t1,t2
integer :: count_rate,count,clock_rate,clock_start,clock_stop
integer :: hours=0,minutes=0,seconds=0
integer :: totaltime=0

interface 
   subroutine seed(mpiid)
     integer mpiid
   end subroutine seed
end interface

!!!!   START MPI 
call MPI_Init ( mpierror )
call MPI_Comm_size ( MPI_COMM_WORLD, mpisize, mpierror )
call MPI_Comm_rank ( MPI_COMM_WORLD, mpiid, mpierror )
if ( mpiid == 0 ) then
   mpiwtime = MPI_Wtime ( )
   write ( *, '(a,i8)' ) '  The number of processes is ', mpisize
end if
!write ( *, '(a,i8,a)' ) '  Process ', mpiid, ' says "Hello, world!"'

!!!!   SIMULATION DATA (provided by module getdata)
if(mpiid==0)call readdata() !simulation data and function chi2
if(stoprun)goto 1001
if(mpisize.gt.1)call bcastdata()
Call MPI_BARRIER(MPI_COMM_WORLD, mpierror )
!!!!   END OF GETDATA MODULE CALL


!!!!   MODEL PARAMETERS
if(mpiid==0)then
print*,'enter number of parameters'
read*,nparams,nparams2
allocate (fitparams(nparams))
print*,'enter filename to read parameter details'
read*,paramsfile
open(10,file=paramsfile,status='OLD',Err=1)
do j=1,nparams
   read(10,*,end=2)fitparams(j)%name,fitparams(j)%range(:)
enddo
close(10)

!!!!   MONTE CARLO PARAMETERS
read*,nblocks,nmoves,nskip,statistics_start
read*,samplingtype
read*,chi2max
read*,ptempering,tempering_rate,tempering_factor
read*,cloning,nclones
read*,writescatter
if(mpisize==1)then
   ptempering=.false.
   cloning=.false.
   ntemperatures=1
   nclones=1
end if
end if
Call MPI_BARRIER(MPI_COMM_WORLD, mpierror )

!!!!  INITIALIZATION 
if(mpisize.gt.1)then
   call bcastparams()
   ntemperatures=mpisize/nclones
   if(ntemperatures*nclones.ne.mpisize)then
      print*,'ERROR: ntemperatures*nclones .ne. mpisize'
      stop
   end if
end if
if(ntemperatures==mpisize)cloning=.false.
if(nclones==mpisize)ptempering=.false.


Call MPI_BARRIER(MPI_COMM_WORLD, mpierror )


temperature=1.
if(ptempering)then
   mpiid_clone = mpiid/ntemperatures       !! row number
   mpiid_temp = mod(mpiid, ntemperatures) !! column number
   call MPI_Comm_split(MPI_COMM_WORLD, mpiid_clone, mpiid_temp, comm_clone, mpierror)
   allocate(temperingorder(0:ntemperatures-1))
   do i=1,mpiid_temp
      temperature=temperature*tempering_factor
   end do
   temperingorder(0)=0
   do i=1,ntemperatures-1
      temperingorder(i)=i
   end do
   allocate (status1(MPI_STATUS_SIZE,4),requests1(4))
   nmoves_ex=0
   exchange_succ=0
end if

!print*,mpiid,mpiid_temp,temperature,ntemperatures,nclones
!goto 1001

Call MPI_BARRIER(MPI_COMM_WORLD, mpierror )
if(mpiid==0)print*,'initializing parameter details'
fitparams(:)%ratio=0.
do i=1,nparams
   fitparams(i)%maxstep=(fitparams(i)%range(2)-fitparams(i)%range(1))/100.
end do
do i=1,nparams
   do j=1,nbins
      fitparams(i)%bin(j,2)=0.
      fitparams(i)%bin(j,1)=fitparams(i)%range(1)+ &
           (j-1)*(fitparams(i)%range(2)-fitparams(i)%range(1))/(nbins-1)
   end do
   fitparams(i)%val=(fitparams(i)%range(2)+fitparams(i)%range(1))/2.
   fitparams(i)%av=0.
   fitparams(i)%sigma=0.
end do
avchi2=0.
sigmachi2=0.
do j=1,nbins
   chi2bin(j,1)=(j-1.)*chi2max/(nbins-1)
end do
chi2bin(:,2)=0.
if(mpiid==0)then
print*,'read the following parameter details:'
print*,'nparams=',nparams
print*,'name                    range'
do i=1,nparams
   print'(A20,4G12.3)',fitparams(i)%name,fitparams(i)%range,fitparams(i)%val,fitparams(i)%expval
end do
end if
Call MPI_BARRIER(MPI_COMM_WORLD, mpierror )
call seed(mpiid)
do i=1,nparams
   if(fitparams(i)%imove==0)cycle
   call moveparams(i)
end do
chi20=chi2()
print'(A25,I4,G12.3)','mpiid/initial chi2',mpiid,chi20
Call MPI_BARRIER(MPI_COMM_WORLD, mpierror )
!goto 1000


!!$if(mpisize.gt.0.and.ptempering.and.mpiid==0)then
!!$   print*,'mpiid/temperingorder:',mpiid,temperingorder
!!$   print*,'mpiid/temperatures',mpiid,temperature
!!$end if

!!$print*,'params=',fitparams(:)%val
!!$
!!$stop


if(writescatter.and.mpiid==0)then
   if(mpiid.le.999)write(ch,'(I3)')mpiid
   if(mpiid.le.99)write(ch,'(I2)')mpiid
   if(mpiid.le.9)write(ch,'(I1)')mpiid
   open(111+mpiid,file='scatter_'//trim(ch))
   write(111,'(A8)')'NAMEAMIA'
   j=0
   do i=1,nparams
      if(fitparams(i)%imove.ne.0)j=j+1
   end do
   write(111,*)j+1   
   do i=1,nparams
      if(fitparams(i)%imove.ne.0)write(111,*)fitparams(i)%name
   end do
   write(111,*)'chi2'
end if


!goto 1001

!!!!    MONTE CARLO 
do iblock=1,nblocks
   icount=-1
   if(mpiid==0)then
      call system_clock(count_rate=clock_rate) !Find the time rate
      call system_clock(count=clock_start)     !Start Timer
   end if
   imoves_tempering=0
   do imove=1,nmoves
      imoves_tempering=imoves_tempering+1
      icount=icount+1
      if(icount==ntemperatures-1)icount=0
      do iskip=1,nskip
         do i=1,nparams
            if(fitparams(i)%imove==0)cycle
            temp1=fitparams(i)%val
            call moveparams(i)
            temp2=chi2()
            select case(samplingtype)
            case(1) !monte-carlo uniform
               pass=evalmv((temp2-chi20)/2./temperature)
            case(2) !monte-carlo radial (most propably this is wrong!)
               x=min(1.,fitparams(i)%val*exp(-temp2/2.)/temp1/exp(-chi20/2.))
               pass=.false.
               if(random().le.x)pass=.true.
            case(11) !uniform
               !x=min(1.,fitparams(i)%val*exp(-temp2/2.)/temp1/exp(-chi20/2.))
               pass=.true.
               if(random().le.x)pass=.true.
            case default
               print*,'UNDEFINIED CHOICE FOR SAMPLINGTYPE'
            end select
            if(pass)then
               chi20=temp2
            else
               fitparams(i)%val=temp1
            end if
            call mcstatistics(i,pass)
            if(iskip==nskip.and.iblock.gt.statistics_start)then
               call binning()
               call special_binning()
            end if
         end do
      end do
      if(writescatter)call scatterplot()
      !Call MPI_BARRIER(MPI_COMM_WORLD, mpierror )
      if(ptempering.and.imoves_tempering==tempering_rate)then 
         imoves_tempering=0
         i1=temperingorder(icount)
         i2=temperingorder(icount+1)
         call evalexchange(i1,i2,pass2)
         call tempering_statistics(pass2)
      end if
      !Call MPI_BARRIER(MPI_COMM_WORLD, mpierror )
   end do !!!! end of monte-carlo block moves
   Call MPI_BARRIER(MPI_COMM_WORLD, mpierror )
   if(iblock.gt.statistics_start)then
      call plotbins()
      call special_plotbins()
      call MPI_BARRIER(MPI_COMM_WORLD,mpierror)
   end if
   call MPI_ALLREDUCE( chi2eval, itemp, 1, MPI_INTEGER, MPI_SUM,MPI_COMM_WORLD,mpierror)
   call MPI_ALLREDUCE( avchi2, temp1, 1, MPI_DOUBLE_PRECISION, MPI_SUM,MPI_COMM_WORLD,mpierror)
   call MPI_ALLREDUCE( sigmachi2, temp2, 1, MPI_DOUBLE_PRECISION, MPI_SUM,MPI_COMM_WORLD,mpierror)
   if(mpiid==0)then
      write(6,*)'############################################'
      write(6,*)'######## MONTE CARLO BLOCK RESULTS #########'
      write(6,*)iblock,iblock*nmoves
      if(iblock.gt.statistics_start)then
         write(6,*)'chi2:',temp1/itemp,sqrt(temp2/itemp-(temp1/itemp)**2)
      else
         write(6,*)'chi2:',chi2()
      end if
      write(6,*)'par.name               val.       maxstep     ratio       average     sigma        neval.'    
   end if
   do i=1,nparams
      if(mpisize.gt.1.and.iblock.gt.statistics_start)then
         temp1=0. ; temp2=0. ; itemp=0
         call MPI_ALLREDUCE( fitparams(i)%av, temp1, 1, MPI_DOUBLE_PRECISION, MPI_SUM,MPI_COMM_WORLD,mpierror)
         call MPI_ALLREDUCE( fitparams(i)%sigma, temp2, 1, MPI_DOUBLE_PRECISION, MPI_SUM,MPI_COMM_WORLD,mpierror)
         call MPI_ALLREDUCE( fitparams(i)%neval, itemp, 1, MPI_INTEGER, MPI_SUM,MPI_COMM_WORLD,mpierror)
      elseif(iblock.gt.statistics_start)then
         temp1=fitparams(i)%av
         temp2=fitparams(i)%sigma
         itemp=fitparams(i)%neval
      end if
      if(mpiid==0)then
         if(fitparams(i)%imove==0)then
            write(6,'(A20,g12.4,2A10)')fitparams(i)%name,fitparams(i)%val,'N/A','N/A' !,fitparams(i)%imove
         elseif(iblock.gt.statistics_start)then
            write(6,'(A20,5g12.4)')fitparams(i)%name,fitparams(i)%val,fitparams(i)%maxstep, &
                 fitparams(i)%nsucc/real(fitparams(i)%nmove),temp1/itemp,sqrt(temp2/itemp - (temp1/itemp)**2)!,&
         elseif(iblock.le.statistics_start)then  
            write(6,'(A20,3g12.4,3A10)')fitparams(i)%name,fitparams(i)%val,fitparams(i)%maxstep, &
                 fitparams(i)%nsucc/real(fitparams(i)%nmove),'N/A','N/A','N/A'
         end if
      end if
   end do
   call MPI_BARRIER(MPI_COMM_WORLD, mpierror)
   if(mpiid==0)then
      call system_clock(count=clock_stop)
      totaltime=totaltime+real(clock_stop-clock_start)/real(clock_rate)
      hours=totaltime/3600
      minutes=(totaltime-hours*3600)/60
      seconds=totaltime-hours*3600-minutes*60
      print'(A15,I4,A1,I2,A1,I2)','ellapsed time=',hours,':',minutes,':',seconds
      itemp=totaltime/iblock
      temp1=(nblocks-iblock)*itemp
      hours=temp1/3600
      minutes=(temp1-hours*3600)/60
      seconds=temp1-hours*3600-minutes*60
      print'(A15,I4,A1,I2,A1,I2)','time remaining~',hours,':',minutes,':',seconds
   end if
   call MPI_BARRIER(MPI_COMM_WORLD, mpierror)
   if(ptempering.and.mpisize.gt.1.and.mpiid==0)then
      print*,'temp. ratio', mpiid,exchange_succ/real(nmoves_ex)
!!$      print*,'order:',mpiid,temperingorder
!!$      print*,'temperature',mpiid,temperature
   end if
end do




goto 1000

1 continue
print*,'error reading from paramsfile '//trim(paramsfile)
goto 1000
2 continue
print*,'error: paramsfile '//trim(paramsfile)//' ended before its time'
goto 1000

1000 continue


if(ptempering)deallocate(temperingorder,status1,requests1)
if(writescatter)close(111+mpiid)
call cleardata()
deallocate(fitparams)

1001 continue

Call MPI_BARRIER(MPI_COMM_WORLD, mpierror )
call MPI_Finalize ( mpierror )


CONTAINS

subroutine mcstatistics(i,x)
  implicit none
  integer,intent(in) :: i
  logical,intent(in) :: x
  double precision :: d=0.01,limit
  limit=(fitparams(i)%range(2)-fitparams(i)%range(1))/10.
  fitparams(i)%nmove=fitparams(i)%nmove+1
  if(x)then
     fitparams(i)%maxstep=fitparams(i)%maxstep*(1.+d)
     fitparams(i)%nsucc=fitparams(i)%nsucc+1
  else
     fitparams(i)%maxstep=fitparams(i)%maxstep*(1.-d)
  end if
  if(fitparams(i)%maxstep.gt.limit)fitparams(i)%maxstep=limit
end subroutine mcstatistics

function evalmv(diff)
  implicit none
  logical evalmv
  double precision, intent(in) :: diff
  double precision exparg,XQ
  PARAMETER (EXPARG=120.D0)
  IF( DIFF .LE. 0.0) THEN  
     evalmv=.true.
  else
     XQ = MAX( -EXPARG , MIN( EXPARG , -DIFF ) )     
     IF( RANDOM() .LT. DEXP( XQ )) THEN                 
        evalmv=.true.                                                 
     ELSE                                                          
        evalmv=.false.                                                 
     ENDIF
  end IF
end function evalmv

subroutine binning()
implicit none
integer i,k
double precision :: d
!!$logical :: first=.true.
!!$if(first)then
!!$   fitparams(:)bins()=0.
!!$   first=.false.
!!$else
!!$   
!!$end if
if(temperature==1.)then
   do i=1,nparams
      if(fitparams(i)%imove==0)cycle
      d=(fitparams(i)%range(2)-fitparams(i)%range(1))/nbins
      k=(fitparams(i)%val-fitparams(i)%range(1))/d
      if(samplingtype.eq.11 .and. (k.ge.1 .and. k.le.nbins) )then
         fitparams(i)%bin(k,2)=fitparams(i)%bin(k,2)+exp(-chi20/2.)
      elseif(k.ge.1 .and. k.le.nbins)then
         fitparams(i)%bin(k,2)=fitparams(i)%bin(k,2)+1.
      end if
      fitparams(i)%neval=fitparams(i)%neval+1
      fitparams(i)%av=fitparams(i)%av+fitparams(i)%val
      fitparams(i)%sigma=fitparams(i)%sigma+fitparams(i)%val**2
    end do
    avchi2=avchi2+chi20
   sigmachi2=sigmachi2+chi20**2
   chi2eval=chi2eval+1
   d=chi2max/nbins
   k=chi20/d
   if(k.ge.1 .and. k.le.nbins)chi2bin(k,2)=chi2bin(k,2)+1
end if
end subroutine binning

subroutine scatterplot()
implicit none
integer i,k
double precision :: d
if(temperature==1.)then
   do i=1,nparams
      if(fitparams(i)%imove==1)write(111,'(f15.3)',advance='no')fitparams(i)%val
   end do
   write(111,'(G15.3)')chi20
end if
end subroutine scatterplot

subroutine initialize_ranges(i,av0,sigma0,n)
implicit none
double precision, intent(in) :: av0,sigma0
integer,intent(in) :: i,n
integer :: j
double precision :: av,sigma
av=av0/n
sigma=sqrt(sigma0/n-av**2)
fitparams(i)%range(1)=av-30*sigma
fitparams(i)%range(2)=av+30*sigma
do j=1,nbins      
   fitparams(i)%bin(j,2)=0.
   fitparams(i)%bin(j,1)=fitparams(i)%range(1)+ &
        (j-1)*(fitparams(i)%range(2)-fitparams(i)%range(1))/(nbins-1)
end do
!!$r1=avchi2-10*sigmachi2
!!$r2=avchi2+10*sigmachi2
!!$do j=1,nbins
!!$   chi2bin(j,1)=r1+(j-1)*(r2-r1)/(nbins-1)
!!$end do
!!$chi2bin(:,2)=0.
end subroutine initialize_ranges

subroutine plotbins()
character*24 f
integer i,j
double precision :: buffer(nbins),norm
call MPI_BARRIER(MPI_COMM_WORLD,mpierror)
do i=1,nparams
   if(fitparams(i)%imove==0)cycle
   call MPI_REDUCE( fitparams(i)%bin(:,2), buffer(:), nbins, MPI_DOUBLE_PRECISION, MPI_SUM,0,MPI_COMM_WORLD,mpierror)
   if(mpiid==0)then
      f=trim(fitparams(i)%name)//'.bin'
      open(11,file=trim(f))
      norm=0.
      do j=1,nbins
         norm=norm+buffer(j)*(fitparams(i)%bin(2,1)-fitparams(i)%bin(1,1))
      end do
      do j=1,nbins
         write(11,*)fitparams(i)%bin(j,1),buffer(j)/norm
      end do
      if(fitparams(i)%exp)then
         write(11,*)
         write(11,*)
         do j=0,10
            write(11,*)fitparams(i)%expval,maxval(buffer)/10*j
         end do
      end if
      close(11)
   end if
end do
call MPI_REDUCE( chi2bin(:,2), buffer(:), nbins, MPI_DOUBLE_PRECISION, MPI_SUM,0,MPI_COMM_WORLD,mpierror)
if(mpiid==0)then
   f='chi2.bin'
   open(11,file=trim(f))
   do j=1,nbins
      write(11,*)chi2bin(j,1),buffer(j)
   end do
   close(11)
end if
call MPI_BARRIER(MPI_COMM_WORLD,mpierror)
end subroutine plotbins

subroutine moveparams(i)
implicit none
integer,intent(in) :: i
double precision :: d,y
if(fitparams(i)%imove==0)then
   print*,'CRITICAL ERROR IN MOVEPARAMS'
   STOP
END if
if(samplingtype==11)then
   y=fitparams(i)%range(1)+random()*abs(fitparams(i)%range(2)-fitparams(i)%range(1))
   fitparams(i)%val=y
else
   d=fitparams(i)%maxstep*(0.5-random())
   y=abs(fitparams(i)%range(2)-fitparams(i)%range(1))
   fitparams(i)%val=fitparams(i)%val+d
   if(fitparams(i)%val.lt.fitparams(i)%range(1))then
      fitparams(i)%val=fitparams(i)%range(2)-mod(abs(fitparams(i)%val-fitparams(i)%range(2)),y)
!   fitparams(i)%val=fitparams(i)%range(1)
   end if
   if(fitparams(i)%val.gt.fitparams(i)%range(2))then
      fitparams(i)%val=fitparams(i)%range(1)+mod(fitparams(i)%val-fitparams(i)%range(1),y)
!   fitparams(i)%val=fitparams(i)%range(2)
   end if
end if
end subroutine moveparams


subroutine bcastparams()
implicit none
integer :: i
double precision :: temp
character*20 :: char
CALL MPI_BCAST(nparams, 1, MPI_INTEGER, 0, MPI_COMM_WORLD, mpierror)
CALL MPI_BCAST(nparams2, 1, MPI_INTEGER, 0, MPI_COMM_WORLD, mpierror)
Call MPI_BARRIER(MPI_COMM_WORLD, mpierror )
if(mpiid.ne.0)allocate(fitparams(nparams))
Call MPI_BARRIER(MPI_COMM_WORLD, mpierror )
do i=1,nparams
   CALL MPI_BCAST(fitparams(i)%name, 20, MPI_CHARACTER, 0, MPI_COMM_WORLD, mpierror)
   CALL MPI_BCAST(fitparams(i)%range, 2, MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, mpierror)
end do
Call MPI_BARRIER(MPI_COMM_WORLD, mpierror )
CALL MPI_BCAST(nblocks, 1, MPI_INTEGER, 0, MPI_COMM_WORLD, mpierror)
CALL MPI_BCAST(samplingtype, 1, MPI_INTEGER, 0, MPI_COMM_WORLD, mpierror)
CALL MPI_BCAST(statistics_start, 1, MPI_INTEGER, 0, MPI_COMM_WORLD, mpierror)
CALL MPI_BCAST(chi2max, 1, MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, mpierror)
CALL MPI_BCAST(nmoves, 1, MPI_INTEGER, 0, MPI_COMM_WORLD, mpierror)
CALL MPI_BCAST(nclones, 1, MPI_INTEGER, 0, MPI_COMM_WORLD, mpierror)
CALL MPI_BCAST(nskip, 1, MPI_INTEGER, 0, MPI_COMM_WORLD, mpierror)
CALL MPI_BCAST(ptempering, 1, MPI_LOGICAL, 0, MPI_COMM_WORLD, mpierror)
CALL MPI_BCAST(cloning, 1, MPI_LOGICAL, 0, MPI_COMM_WORLD, mpierror)
CALL MPI_BCAST(tempering_factor, 1, MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, mpierror)
CALL MPI_BCAST(tempering_rate, 1, MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, mpierror)
CALL MPI_BCAST(writescatter, 1, MPI_LOGICAL, 0, MPI_COMM_WORLD, mpierror)
!print*,mpiid,' received params',nparams
!print*,mpiid,nparams,fitparams(:)%name
end subroutine bcastparams


  subroutine evalexchange(i1,i2,pass2)
    integer,intent(in) :: i1,i2
    !real*8 buffer1(nparams_1),buffer2(nparams_2)
    double precision :: temp,prop,temp2
    integer :: i,j,ii,jj,source,dest,tag
    integer status(MPI_STATUS_SIZE)
    integer :: pass2,ncall=0
    ncall=ncall+1
    pass2=-1
    if(i1.lt.0.or.i1.gt.ntemperatures-1.or.i2.lt.0.or.i2.gt.ntemperatures-1.or.i1==i2)then
       print*,'error in evalexchange',i1,i2,mpiid_temp,ntemperatures
       print*,'ncall=',ncall
       print*,'mpiid,temperature,temperingorder',temperature,temperingorder
       stop
    end if
    if(mpiid_temp==i1 .or. mpiid_temp==i2)then
       pass2=0
       if(mpiid_temp==i1)then
          dest=i2
          source=i2
       end if
       if(mpiid_temp==i2)then
          dest=i1
          source=i1
       end if
       requests1(1)=1+mpiid_clone*4
       requests1(2)=2+mpiid_clone*4
       requests1(3)=3+mpiid_clone*4
       requests1(4)=4+mpiid_clone*4
       tag=1
       call mpi_isend(chi20, 1, MPI_DOUBLE_PRECISION, dest, tag , &
            comm_clone, requests1(1), mpierror)
       call mpi_irecv(temp, 1, MPI_DOUBLE_PRECISION, source,tag , &
            comm_clone, requests1(2), mpierror)
       tag=2
       call mpi_isend(temperature, 1, MPI_DOUBLE_PRECISION, dest, tag , &
            comm_clone, requests1(3), mpierror)
       call mpi_irecv(temp2, 1, MPI_DOUBLE_PRECISION, source,tag , &
            comm_clone, requests1(4), mpierror)
       call MPI_waitall(4,requests1,status1,mpierror)
       if(mpiid_temp==i1)then    
          prop=min(1.D0,exp((chi20-temp)* &
               (1./2./temperature-1./2./temp2)))
          if(random().lt.prop)pass2=1
          call MPI_Send(pass2, 1, MPI_INTEGER, i2, tag, comm_clone,mpierror);
       end if
       if(mpiid_temp==i2)then
          call mpi_recv(pass2,1,MPI_INTEGER,i1,tag,comm_clone,status,mpierror)
       end if
       !print*,mpiid,pass2

       if(pass2==1)then
          temperature=temp2
          chi20=chi2()
          !chi20=temp
          if(mpiid_temp==i1)then
             do i=0,ntemperatures-1
                if(temperingorder(i)==i1)ii=i
                if(temperingorder(i)==i2)jj=i
             end do
             temperingorder(ii)=i2
             temperingorder(jj)=i1
          end if
       end if
    end if
    Call MPI_BARRIER(comm_clone, mpierror )
    CALL MPI_BCAST(temperingorder(0:ntemperatures-1), ntemperatures, MPI_INTEGER, i1, comm_clone, mpierror)
  end subroutine evalexchange

  subroutine tempering_statistics(pass2)
    integer :: pass2
    if(pass2==-1)return
    if(pass2==1)exchange_succ=exchange_succ+1
    nmoves_ex=nmoves_ex+1
  end subroutine tempering_statistics


end program


 subroutine seed(mpiid)
   implicit none
   integer i,j,k,count, COUNT_RATE, COUNT_MAX
   integer, allocatable :: seeds(:)
   integer  mpiid
   real*8 ranval
   !if(mpiid==0)write(6,*)'*** random number generator initialization ***'
   call random_seed (size=k)
      allocate (seeds(k))
      j=0
      do i=1,100000*k
         call system_clock(count, COUNT_RATE, COUNT_MAX )
         if(mod(i,100000)==0)then
            j=j+1
            seeds(j)=mod(count,count_rate)*(mpiid+1)
         endif
      enddo
      !if(mpiid==0)write(6,*)mpiid,' used random seeds:',seeds
      !print*,mpiid,'seed',seeds
      call random_seed(PUT=seeds(1:k))
      do i=1,100
         call random_number(ranval)
      enddo
      deallocate(seeds)
 end subroutine seed
